@@ -26,7 +26,8 @@ from .proto_handlers import load_protocol_handler
 from .utils.connect import connect_to_database, dataframe_to_sql, db_conn_flavor
 from .utils.csp_connect import connect_to_csp
 from .sql.misc import sql_enquote_id
-from .utils.df_utils import CloseAfterIteration, load_dataframe_from_sql, LoadStrategy
+from .utils.df_utils import CloseAfterIteration, load_dataframe_from_sql, LoadStrategy, coerce_to_dataframe, \
+    convertible_to_dataframe
 from .utils.file_utils import write_stream_with_read_on_close, connect_to_filesystem, adapt_fsspec_storage_options
 from .utils.format_sniffer import find_url_extension, detect_format, FORMAT_ALIASES_WRITE_OPTIONS
 from .utils.nosql import connect_to_nosql
@@ -77,7 +78,7 @@ class DSLibrary(object):
                         methods to MLFlow.
         """
         self._params = None
-        self._spec_ = spec or {}
+        self._spec_ = self._adjust_spec(spec or {})
         self._env = _env
         spec_wo_data = dict(self._spec)
         spec_wo_data.pop("data", None)
@@ -91,6 +92,12 @@ class DSLibrary(object):
         self._mlflow_run = None
         self._setup_code_paths()
 
+    def _adjust_spec(self, spec: dict) -> dict:
+        """
+        Derived classes may adjust the specification before it is processed.
+        """
+        return spec
+
     @property
     def _spec(self) -> dict:
         if hasattr(THREAD_SPEC, "value"):
@@ -615,7 +622,7 @@ class DSLibrary(object):
             db_conn = self.get_sql_connection(sql_source, database=kwargs.get("database"))
             if not sql_flavor:
                 sql_flavor = db_conn_flavor(db_conn)
-            content = _coerce_to_dataframe(content)
+            content = coerce_to_dataframe(content)
             try:
                 table = sql_table or resource_name
                 for sql, params in dataframe_to_sql(content, table_name=table, append=append, flavor=sql_flavor):
@@ -628,7 +635,7 @@ class DSLibrary(object):
             raise NotImplementedError("NoSQL write of columnar data is not yet implemented")
         # we detect numpy arrays, series, etc., and save them as a dataframe with one column
         if isinstance(content, (pandas.Series, numpy.ndarray, pandas.Index)):
-            content = _coerce_to_dataframe(content)
+            content = coerce_to_dataframe(content)
         # send (pandas) DataFrame as CSV/etc.
         df_format = format if format in PANDAS_WRITE_FORMATS else "csv"
         if hasattr(content, PANDAS_WRITE_FORMATS[df_format]):
@@ -867,7 +874,7 @@ class DSLibrary(object):
         if hasattr(content, "columns"):
             return self.write_resource(uri, content, **kwargs)
         if isinstance(content, (pandas.Series, numpy.ndarray, pandas.Index)):
-            content = _coerce_to_dataframe(content)
+            content = coerce_to_dataframe(content)
         # try to force into a dataframe
         df_args = {}
         if "columns" in kwargs:
@@ -949,6 +956,7 @@ class DSLibrary(object):
         sql_source = open_args.pop("sql_source", None)
         sql_table = open_args.pop("sql_table", None)
         sql_sql = open_args.pop("sql", None)
+        sql_parameters = open_args.pop("sql_parameters", None)
         if sql_source or sql_sql or sql_table:
             if sniff_only:
                 open_args["sniff_only"] = True
@@ -961,14 +969,16 @@ class DSLibrary(object):
                 else:
                     sql_table = resource_name
             if sql_table:
-                # NOTE: 'or resource_name' is backup for table name
+                # table requested, generate SQL to read it
                 flavor = db_conn_flavor(db_conn)
-                sql_sql = f"SELECT * from {sql_enquote_id(sql_table or resource_name, dotted=True, mode=flavor)}"
+                sql_sql = f"SELECT * from {sql_enquote_id(sql_table, dotted=True, mode=flavor)}"
+                sql_parameters = None
             if sniff_only:
                 connect_strategy = db_conn
                 return LoadStrategy(opener=connect_strategy.opener, open_args=connect_strategy.open_args,
-                                    read_method=load_dataframe_from_sql, use_dask=use_dask, sql=sql_sql)
-            return load_dataframe_from_sql(db_conn, sql_sql, use_dask)
+                                    read_method=load_dataframe_from_sql, use_dask=use_dask, sql=sql_sql,
+                                    sql_parameters=sql_parameters)
+            return load_dataframe_from_sql(db_conn, sql=sql_sql, parameters=sql_parameters, use_dask=use_dask)
         # detect NoSQL
         nosql_source = open_args.pop("nosql_source", None)
         nosql_collection = open_args.pop("nosql_collection", None)
@@ -1143,6 +1153,47 @@ class DSLibrary(object):
             if "lines" not in format_options:
                 format_options["lines"] = True
 
+    def sql_select(self, sql: str, parameters: list=None, engine: str=None, data=None, table_loader: callable=None):
+        """
+        Run an SQL 'SELECT' statement against various sources of data.  By default, tables are just filenames, but you
+        can query anything else by specifying 'engine', 'data' or 'table_loader'.
+
+        :param sql:                 An SQL 'SELECT' statement, i.e. "SELECT * from mytable".
+        :param parameters:          Optional parameters, in case the SQL is parameterized.
+        :param engine:              The name of an SQL engine.  See get_sql_connection().  If specified the SQL will be
+                                    sent to this engine.
+        :param data:                Allows SQL to query this data.  If a DataFrame is given, all table names will refer
+                                    to it.  If a {str -> DataFrame} is given, the named tables will be queried.
+        :param table_loader:        Allows SQL to span multiple sources.  Supply a method that accepts a tuple to
+                                    specify the qualified table name, and which returns a DataFrame.
+        :return:        A DataFrame.
+        """
+        n_sources = bool(engine) + bool(table_loader) + bool(data is not None)
+        if n_sources > 1:
+            raise DSLibraryException("specify only one of 'engine', 'data' or 'table_loader'")
+        if not n_sources:
+            # default: assume tables are filenames
+            def default_table_loader(table_spec: tuple):
+                return self.load_dataframe(table_spec[-1])
+            table_loader = default_table_loader
+        if engine:
+            return self.load_dataframe(engine, sql=sql, sql_parameters=parameters)
+        # a collection of tables
+        # NOTE: the '.sql' package is big, so we load it here, only if it is used
+        from .sql.q_pandas import sql_for_tables
+        if hasattr(data, "get") and hasattr(data, "values") and all(convertible_to_dataframe(d) for d in data.values()):
+            return sql_for_tables(sql, tables=data)
+        # one table
+        if data is not None:
+            try:
+                data = coerce_to_dataframe(data)
+            except NotImplementedError:
+                pass
+            return sql_for_tables(sql, fallback_data=data)
+        # custom table loader
+        if table_loader:
+            return sql_for_tables(sql, table_loader=table_loader)
+
     def load_pickled_model(self, part: str=None):
         """
         Un-pickle a saved model from the default model binary, or from one of its named parts.
@@ -1236,23 +1287,6 @@ LOCAL_SPECS_SCHEMA = {
 }
 
 
-def _coerce_to_dataframe(content):
-    """
-    Turn things into dataframes.
-    """
-    if hasattr(content, "columns") and hasattr(content, "itertuples"):
-        return content
-    elif hasattr(content, "__iter__"):
-        content = pandas.DataFrame(content)
-        if len(content.columns) and not isinstance(content.columns[0], str):
-            # numeric column names export ambiguously to CSV
-            # TODO align this convention with pandas methods which generate (maybe) "Col_1", etc.
-            content.columns = [f"col{n}" for n in range(1, len(content.columns)+1)]
-        return content
-    else:
-        raise NotImplementedError(f"cannot convert {type(content)} to dataframe")
-
-
 def _read_hdf(filename: str, **kwargs):
     """
     Not all HDF files can stream.  This method falls back to loading the whole dataframe at once, but presenting
