@@ -1,6 +1,7 @@
 from urllib.parse import urlparse, unquote, urlunparse, quote
 import re
 import typing
+import json
 
 
 def connect_to_database(uri: str, library: str=None, for_write: bool=False, **kwargs):
@@ -190,6 +191,31 @@ def connect_sqlite(host, database=None, for_write: bool=False, sniff_only: bool=
     return conn
 
 
+def connect_bigquery(host=None, database=None, for_write: bool=False, sniff_only: bool=False, **kwargs):
+    try:
+        from .bigq import create_bgq_connection
+    except ImportError:
+        return
+    service_account = None
+    project = None
+    if "service_account" in kwargs:
+        service_account = kwargs.pop("service_account")
+    elif isinstance(database, str) and database.startswith("{"):
+        service_account = json.loads(database)
+    elif isinstance(database, dict):
+        service_account = database
+    if "project" in kwargs:
+        project = kwargs.pop("project")
+    params = dict(service_account=service_account, project=project, **kwargs)
+    if not for_write:
+        params["read_only"] = True
+    if sniff_only:
+        return ConnectStrategy(opener=create_bgq_connection, open_args=params)
+    conn = create_bgq_connection(**params)
+    conn._flavor = "bigquery"
+    return conn
+
+
 def _set_conn_read_only(conn):
     """
     Try to enforce a read-only connection
@@ -278,7 +304,6 @@ METHODS = {
     "pymysql": connect_pymysql,
     "psycopg2": connect_psycopg2,
     "pymssql": connect_pymssql,
-    "sqlite": connect_sqlite
+    "sqlite": connect_sqlite,
+    "bigquery": connect_bigquery,
 }
-
-# TODO BigQuery
