@@ -5,6 +5,7 @@ import typing
 import os
 import re
 import csv
+import json
 from collections import namedtuple, defaultdict
 
 FileFormatDescription = namedtuple("FileFormatDescription", "format read_args size avg_line_size")
@@ -117,7 +118,7 @@ def detect_format(
     # - some formats are not sniffed so we don't sample them
     if sampler and format not in ("yaml", "xlsx", "xls", "parquet") and orig_format not in ("txt",):
         sample = sampler() if hasattr(sampler, "__call__") else sampler
-        if not format and looks_like_json(sample, size):
+        if not format and looks_like_json(sample, size, other_args=read_args):
             format = "json"
         elif not format and looks_like_csv(sample):
             format = "csv"
@@ -219,7 +220,7 @@ def looks_like_csv(sample):
         return TEXT_FILE_DELIMITER
 
 
-def looks_like_json(sample, size: int=None):
+def looks_like_json(sample, size: int=None, other_args: dict=None):
     """
     Detect fairly obvious JSON format.
     """
@@ -239,9 +240,18 @@ def looks_like_json(sample, size: int=None):
     if re.search(r'[^\x20-\x7F\r\t\n]', sample):
         return False
     # if we know the size and our sample goes to the end we can check that as well
-    if size and len(sample) == size:
+    if size and len(sample) >= size:
         if not sample.endswith("}") and not sample.endswith("]"):
             return False
+    # determine whether this is 'lines' format, i.e. one json block per line
+    if other_args is not None and "\n" in sample:
+        lines = sample.split("\n")
+        try:
+            json.loads(lines[0].strip(","))
+            # first line is a valid JSON chunk - looks like 'lines' format
+            other_args["lines"] = True
+        except:
+            pass
     # looks like JSON, although it could still be a CSV with bracketed columns like "[col1], [col2]" for which
     #   we have an incomplete sample
     return True
