@@ -128,7 +128,7 @@ class TestFront(unittest.TestCase):
                 self.assertEqual(kwargs["username"], "u")
                 class DbCursor(object):
                     description = [("x", None)]
-                    def execute(self, sql):
+                    def execute(self, sql, parameters=None):
                         log.append(sql)
                     def __iter__(self):
                         return iter([(1,), (2,)])
@@ -162,12 +162,12 @@ class TestFront(unittest.TestCase):
         # series
         dsl.write_resource("x", pandas.Series([1, 2, 3]))
         assert log[0][0] == "x"
-        self.assertEqual(log[0][1].getvalue(), 'col1\n1\n2\n3\n')
+        self.assertEqual(log[0][1].getvalue(), 'col_1\n1\n2\n3\n')
         log.clear()
         # numpy array
         dsl.write_resource("x", numpy.array([1, 2, 3]))
         assert log[0][0] == "x"
-        self.assertEqual(log[0][1].getvalue(), 'col1\n1\n2\n3\n')
+        self.assertEqual(log[0][1].getvalue(), 'col_1\n1\n2\n3\n')
         log.clear()
         # dataframe
         dsl.write_resource("x", pandas.DataFrame({"y": [1, 2, 3]}))
@@ -252,7 +252,7 @@ class TestFront(unittest.TestCase):
             def get_sql_connection(self, resource_name: str, for_write: bool=False, **kwargs):
                 class DbCursor(object):
                     description = [("x", None)]
-                    def execute(self, sql):
+                    def execute(self, sql, parameters=None):
                         log.append(sql)
                     def __iter__(self):
                         return iter([(1,), (2,)])
@@ -288,7 +288,7 @@ class TestFront(unittest.TestCase):
                 assert kwargs["custom1"] == 123
                 class DbCursor(object):
                     description = [("x", None)]
-                    def execute(self, sql):
+                    def execute(self, sql, parameters=None):
                         pass
                     def __iter__(self):
                         return iter([])
@@ -599,8 +599,8 @@ class TestFront(unittest.TestCase):
         MyInst().write_resource("my_pg", pandas.Series([1, 2, 3]), append=False, sql_table="tbl1", database="db4")
         self.assertEqual(log, [
             ('DROP TABLE IF EXISTS tbl1', []),
-            ('CREATE TABLE IF NOT EXISTS tbl1 (col1 INTEGER)', []),
-            ('INSERT INTO tbl1 (col1) VALUES (%s), (%s), (%s)', [1, 2, 3])
+            ('CREATE TABLE IF NOT EXISTS tbl1 (col_1 INTEGER)', []),
+            ('INSERT INTO tbl1 (col_1) VALUES (%s), (%s), (%s)', [1, 2, 3])
         ])
 
     def test_load_dataframe__dask_autodetect(self):
@@ -803,7 +803,7 @@ class TestFront(unittest.TestCase):
                 log.append(("connect", resource_name, for_write, kwargs))
                 class DbCursor(object):
                     description = [("x", None)]
-                    def execute(self, sql):
+                    def execute(self, sql, parameters=None):
                         log.append(sql)
                     def __iter__(self):
                         return iter([(1,), (2,)])
