@@ -3,7 +3,7 @@ import tempfile
 import shutil
 import unittest
 import dslibrary
-from dslibrary import ENV_DSLIBRARY_TARGET, ENV_DSLIBRARY_SPEC, run_model_method, DSLibrary
+from dslibrary import ENV_DSLIBRARY_TARGET, run_model_method, DSLibrary, PTN_DATA_FILES, RECOGNIZED_FILE_EXTENSIONS
 from dslibrary.transport.to_local import DSLibraryLocal
 from dslibrary.transport.to_mmlibrary import DSLibraryViaMMLibrary
 from dslibrary.transport.to_rest import DSLibraryViaREST
@@ -11,7 +11,7 @@ from dslibrary.transport.to_volume import DSLibraryViaVolume
 from tests.t_utils import reset_env
 
 
-class TestRunModelMethod(unittest.TestCase):
+class TestPackageMisc(unittest.TestCase):
 
     def test_run_model_method(self):
         def m(a: int):
@@ -20,6 +20,19 @@ class TestRunModelMethod(unittest.TestCase):
         r = run_model_method(m, dsl=dsl)
         assert r == 5
 
+    def test_PTN_DATA_FILES(self):
+        assert PTN_DATA_FILES.search(".csv")
+        assert PTN_DATA_FILES.search(".csv.gzip")
+        assert PTN_DATA_FILES.search(".xlsx")
+        assert not PTN_DATA_FILES.search(".abc")
+        assert PTN_DATA_FILES.search("abc.csv")
+        assert not PTN_DATA_FILES.search("abc.not_csv")
+        assert PTN_DATA_FILES.search("path/abc.json")
+
+    def test_RECOGNIZED_FILE_EXTENSIONS(self):
+        assert "csv" in RECOGNIZED_FILE_EXTENSIONS
+        assert "zzz" not in RECOGNIZED_FILE_EXTENSIONS
+
 
 class TestMMFront(unittest.TestCase):
 
